<?php

namespace Drupal\entity_theme_engine\Normalizer;


use Drupal\file\Entity\File;
use Drupal\system\Entity\Menu;
use Drupal\Core\Menu\MenuTreeParameters;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Menu\MenuLinkTreeInterface;
use Drupal\Core\Menu\MenuActiveTrailInterface;
use Drupal\taxonomy\Entity\Vocabulary;

class EntityReferenceItemNormalizer extends FieldItemNormalizer {

  /**
   * The interface or class that this Normalizer supports.
   *
   * @var string
   */
  protected $supportedInterfaceOrClass = [
    'Drupal\Core\Field\Plugin\Field\FieldType\EntityReferenceItem',
  ];

  /**
   * @var \Drupal\Core\Menu\MenuActiveTrailInterface
   */
  protected $menu_active_trail;
  
  /**
   * @var \Drupal\Core\Menu\MenuLinkTreeInterface
   */
  protected $menu_tree;
  
  /**
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_manager
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager, MenuActiveTrailInterface $menu_active_trail, MenuLinkTreeInterface $menu_tree) {
    $this->entityTypeManager = $entityTypeManager;
    $this->menu_active_trail = $menu_active_trail;
    $this->menu_tree = $menu_tree;
  }
  /**
   * {@inheritdoc}
   */
  public function normalize($field, $format = NULL, array $context = []) {
    $data = parent::normalize($field, $format, $context);
    $data['#cache_tags'] = ["{$field->entity->getEntityTypeId()}:{$field->entity->id()}"];
    $data['label'] = $field->entity->label();
    $data['url'] = $field->entity->hasLinkTemplate('canonical')?$field->entity->toUrl()->toString():"";
    $data['uuid'] = $field->entity->uuid();
    try {
      $data['render'] = $this->entityTypeManager->getViewBuilder($field->entity->getEntityTypeId())->view($field->entity);
    } catch (\Exception $e) {
    }
    $field_definition = $context['field_definition'];
    $settings = $field_definition->getFieldStorageDefinition()->getSettings();
    if(!empty($settings['target_type'])) {
      $target_type = $settings['target_type'];
      switch ($target_type) {
        case 'media':
          $media_source = $field->entity->getSource()->getSourceFieldValue($field->entity);
          $file = $this->entityTypeManager->getStorage('file')->load($media_source);
          if($file && $file instanceof File) {
            if($field->entity->bundle() == 'image') {
              $data['styles'] = $this->getImageStylesVariables($file);
            }
            $media_source = file_create_url($file->getFileUri());
          }
          $data['media_source'] = $media_source;
          break;
        case 'image':
          $data['styles'] = $this->getImageStylesVariables($field->entity);
          break;
        case 'menu':
          $data['data'] = $this->getMenuData($field->entity);
          break;
        case 'taxonomy_vocabulary':
          $data['data'] = $this->getVocabularyData($field->entity);
          break;
      }
    }
    return $data;
  }
  /**
   * 
   * @param Vocabulary $vocabulary
   * @return array
   */
  public function getVocabularyData(Vocabulary $vocabulary) {
    return [];
  }
  /**
   * 
   * @param Menu $menu
   * @return array
   */
  public function getMenuData(Menu $menu) {
    $parameters = new MenuTreeParameters();
    $parameters->onlyEnabledLinks();
    
    $menu_active_trail = $this->menu_active_trail->getActiveTrailIds($menu->id());
    $parameters->setActiveTrail($menu_active_trail);
    
    $links = $this->menu_tree->load($menu->id(), $parameters);
    $manipulators = [
      ['callable' => 'menu.default_tree_manipulators:checkNodeAccess'],
      ['callable' => 'menu.default_tree_manipulators:checkAccess'],
      ['callable' => 'menu.default_tree_manipulators:generateIndexAndSort'],
    ];
    $links = $this->menu_tree->transform($links, $manipulators);
    $data = static::processMenuLink($links);
    return $data;
  }
  /**
   * 
   * @param array $links
   * @return array|NULL[][]|array
   */
  protected  function processMenuLink($links) {
    if(empty($links)) return [];
    $data = [];
    foreach ($links as $key => $item) {
      $data[$key] = [
        'url' => $item->link->getUrlObject()->toString(),
        'title' => $item->link->getTitle(),
        'options' => $item->link->getOptions(),
        'active' => $item->inActiveTrail,
      ];
      if($item->hasChildren) {
        $data[$key]['subtree'] = $this->processMenuLink($item->subtree);
      }
    }
    return $data;
  }
}
